
#include "NumericEvalValue.hpp"

namespace Porygon::Evaluation {
    const shared_ptr<NumericEvalValue> NumericEvalValue::operator+(const shared_ptr<NumericEvalValue> &b) const {
        if (this->IsFloat()) {
            if (b->IsFloat()) {
                return make_shared<FloatEvalValue>(this->GetFloatValue() + b->GetFloatValue());
            } else {
                return make_shared<FloatEvalValue>(this->GetFloatValue() + b->GetIntegerValue());
            }
        } else {
            if (b->IsFloat()) {
                return make_shared<FloatEvalValue>(this->GetIntegerValue() + b->GetFloatValue());
            } else {
                return make_shared<IntegerEvalValue>(this->GetIntegerValue() + b->GetIntegerValue());
            }
        }
    }

    const shared_ptr<NumericEvalValue> NumericEvalValue::operator-(const shared_ptr<NumericEvalValue> &b) const {
        if (this->IsFloat()) {
            if (b->IsFloat()) {
                return make_shared<FloatEvalValue>(this->GetFloatValue() - b->GetFloatValue());
            } else {
                return make_shared<FloatEvalValue>(this->GetFloatValue() - b->GetIntegerValue());
            }
        } else {
            if (b->IsFloat()) {
                return make_shared<FloatEvalValue>(this->GetIntegerValue() - b->GetFloatValue());
            } else {
                return make_shared<IntegerEvalValue>(this->GetIntegerValue() - b->GetIntegerValue());
            }
        }
    }

    const shared_ptr<NumericEvalValue> NumericEvalValue::operator*(const shared_ptr<NumericEvalValue> &b) const {
        if (this->IsFloat()) {
            if (b->IsFloat()) {
                return make_shared<FloatEvalValue>(this->GetFloatValue() * b->GetFloatValue());
            } else {
                return make_shared<FloatEvalValue>(this->GetFloatValue() * b->GetIntegerValue());
            }
        } else {
            if (b->IsFloat()) {
                return make_shared<FloatEvalValue>(this->GetIntegerValue() * b->GetFloatValue());
            } else {
                return make_shared<IntegerEvalValue>(this->GetIntegerValue() * b->GetIntegerValue());
            }
        }
    }

    const shared_ptr<NumericEvalValue> NumericEvalValue::operator/(const shared_ptr<NumericEvalValue> &b) const {
        if (this->IsFloat()) {
            if (b->IsFloat()) {
                return make_shared<FloatEvalValue>(this->GetFloatValue() / b->GetFloatValue());
            } else {
                return make_shared<FloatEvalValue>(this->GetFloatValue() / b->GetIntegerValue());
            }
        } else {
            if (b->IsFloat()) {
                return make_shared<FloatEvalValue>(this->GetIntegerValue() / b->GetFloatValue());
            } else {
                return make_shared<IntegerEvalValue>(this->GetIntegerValue() / b->GetIntegerValue());
            }
        }
    }

    const bool NumericEvalValue::operator==(EvalValue *b) const {
        if (b->GetTypeClass() != TypeClass::Number)
            return false;
        auto numVal = (NumericEvalValue *) b;
        if (this->IsFloat() != numVal->IsFloat())
            return false;

        if (this->IsFloat()) {
            return this->EvaluateFloat() == numVal->EvaluateFloat();
        } else {
            return this->EvaluateInteger() == numVal->EvaluateInteger();
        }
    }

    const shared_ptr<BooleanEvalValue> NumericEvalValue::operator<(const shared_ptr<NumericEvalValue> &b) const {
        if (this->IsFloat()) {
            if (b->IsFloat()) {
                return make_shared<BooleanEvalValue>(this->GetFloatValue() < b->GetFloatValue());
            } else {
                return make_shared<BooleanEvalValue>(this->GetFloatValue() < b->GetIntegerValue());
            }
        } else {
            if (b->IsFloat()) {
                return make_shared<BooleanEvalValue>(this->GetIntegerValue() < b->GetFloatValue());
            } else {
                return make_shared<BooleanEvalValue>(this->GetIntegerValue() < b->GetIntegerValue());
            }
        }
    }

    const shared_ptr<BooleanEvalValue> NumericEvalValue::operator<=(const shared_ptr<NumericEvalValue> &b) const {
        if (this->IsFloat()) {
            if (b->IsFloat()) {
                return make_shared<BooleanEvalValue>(this->GetFloatValue() <= b->GetFloatValue());
            } else {
                return make_shared<BooleanEvalValue>(this->GetFloatValue() <= b->GetIntegerValue());
            }
        } else {
            if (b->IsFloat()) {
                return make_shared<BooleanEvalValue>(this->GetIntegerValue() <= b->GetFloatValue());
            } else {
                return make_shared<BooleanEvalValue>(this->GetIntegerValue() <= b->GetIntegerValue());
            }
        }
    }

    const shared_ptr<BooleanEvalValue> NumericEvalValue::operator>(const shared_ptr<NumericEvalValue> &b) const {
        if (this->IsFloat()) {
            if (b->IsFloat()) {
                return make_shared<BooleanEvalValue>(this->GetFloatValue() > b->GetFloatValue());
            } else {
                return make_shared<BooleanEvalValue>(this->GetFloatValue() > b->GetIntegerValue());
            }
        } else {
            if (b->IsFloat()) {
                return make_shared<BooleanEvalValue>(this->GetIntegerValue() > b->GetFloatValue());
            } else {
                return make_shared<BooleanEvalValue>(this->GetIntegerValue() > b->GetIntegerValue());
            }
        }
    }

    const shared_ptr<BooleanEvalValue> NumericEvalValue::operator>=(const shared_ptr<NumericEvalValue> &b) const {
        if (this->IsFloat()) {
            if (b->IsFloat()) {
                return make_shared<BooleanEvalValue>(this->GetFloatValue() >= b->GetFloatValue());
            } else {
                return make_shared<BooleanEvalValue>(this->GetFloatValue() >= b->GetIntegerValue());
            }
        } else {
            if (b->IsFloat()) {
                return make_shared<BooleanEvalValue>(this->GetIntegerValue() >= b->GetFloatValue());
            } else {
                return make_shared<BooleanEvalValue>(this->GetIntegerValue() >= b->GetIntegerValue());
            }
        }
    }
}