#ifndef PORYGONLANG_TABLEEVALVALUE_HPP
#define PORYGONLANG_TABLEEVALVALUE_HPP
#include <utility>
#include <unordered_map>
#include "EvalValue.hpp"

using namespace std;

namespace Porygon::Evaluation {
    class TableEvalValue : public EvalValue {
        shared_ptr<unordered_map<uint32_t, shared_ptr<EvalValue>>> _table;
        size_t _hash;

        explicit TableEvalValue(shared_ptr<unordered_map<uint32_t, shared_ptr<EvalValue>>> table, size_t hash) {
            _table = std::move(table);
            _hash = hash;
        }

    public:
        explicit TableEvalValue(shared_ptr<unordered_map<uint32_t, shared_ptr<EvalValue>>> table) {
            _table = std::move(table);
            _hash = rand();
        }

        const TypeClass GetTypeClass() const final {
            return TypeClass::Table;
        }

        const size_t GetHashCode() const final {
            return _hash;
        }

        const bool operator==(EvalValue *b) const final {
            return this->_hash == b->GetHashCode();
        }

        const shared_ptr<EvalValue> Clone() const final {
            return shared_ptr<EvalValue>(new TableEvalValue(_table, _hash));
        }

        const shared_ptr<EvalValue> IndexValue(EvalValue *val) const final {
            auto hash = val->GetHashCode();
            return this->_table->at(hash);
        }

        const shared_ptr<EvalValue> IndexValue(uint32_t hash) const final {
            return this->_table->at(hash);
        }

        const shared_ptr<EvalValue> IndexValue(const char *val) const {
            auto hash = Utilities::HashedString::ConstHash(val);
            return this->_table->at(hash);
        }

        void SetIndexValue(EvalValue *key, const shared_ptr<EvalValue> &value) const final {
            auto hash = key->GetHashCode();
            this->_table->at(hash) = value;
        }
    };
}

#endif //PORYGONLANG_TABLEEVALVALUE_HPP
