
#include "EvaluationScope.hpp"
#include <memory>

namespace Porygon::Evaluation {
    EvaluationScope::EvaluationScope(unordered_map<uint32_t, shared_ptr<EvalValue>> *scriptVariables,
                                     int localVariableCount) {
        _scriptScope = scriptVariables;
        _localScope = unordered_map<uint64_t, shared_ptr<EvalValue>>(localVariableCount);
    }

    void EvaluationScope::CreateVariable(const BoundVariableKey *key, const shared_ptr<EvalValue> &value) {
        if (key->GetScopeId() == 0) {
            _scriptScope->at(key->GetIdentifier()) = value;
        } else {
            auto insert = _localScope.insert({key->GetHash(), value});
            if (!insert.second) {
                _localScope[key->GetHash()] = value;
            }
        }
    }

    void EvaluationScope::SetVariable(const BoundVariableKey *key, const shared_ptr<EvalValue> &value) {
        if (key->GetScopeId() == 0) {
            _scriptScope->at(key->GetIdentifier()) = value;
        } else {
            _localScope[key->GetHash()] = value;
        }
    }

    shared_ptr<EvalValue> EvaluationScope::GetVariable(const BoundVariableKey *key) {
        if (key->GetScopeId() == 0) {
            return _scriptScope->at(key->GetIdentifier());
        } else {
            return _localScope[key->GetHash()];
        }
    }
}