
#ifndef PORYGONLANG_EVALUATOR_HPP
#define PORYGONLANG_EVALUATOR_HPP

#include <string>
#include "../Binder/BoundStatements/BoundStatement.hpp"
#include "EvalValues/EvalValue.hpp"
#include "EvalValues/NumericEvalValue.hpp"
#include "EvalValues/StringEvalValue.hpp"
#include "EvalValues/StringEvalValue.hpp"
#include "EvalValues/ScriptFunctionEvalValue.hpp"
#include "EvaluationScope/EvaluationScope.hpp"

using namespace std;

namespace Porygon::Evaluation{
    class Evaluator {
        shared_ptr<EvalValue> _returnValue;
        unordered_map<uint32_t, shared_ptr<EvalValue>>* _scriptVariables;
        bool _hasReturned;
        shared_ptr<EvalValue> _lastValue;

        //Porygon::Script* _scriptData;
        shared_ptr<EvaluationScope> _evaluationScope;

        void EvaluateStatement(const BoundStatement* statement);
        void EvaluateBlockStatement(const BoundBlockStatement *statement);
        void EvaluateExpressionStatement(const BoundExpressionStatement* statement);
        void EvaluateAssignmentStatement(const BoundAssignmentStatement* statement);
        void EvaluateIndexAssignmentStatement(const BoundIndexAssignmentStatement* statement);
        void EvaluateFunctionDeclarationStatement(const BoundFunctionDeclarationStatement *statement);
        void EvaluateReturnStatement(const BoundReturnStatement *statement);
        void EvaluateConditionalStatement(const BoundConditionalStatement *statement);

        const shared_ptr<EvalValue> EvaluateExpression(const BoundExpression *expression);
        const shared_ptr<NumericEvalValue> EvaluateIntegerExpression(const BoundExpression *expression);
        const shared_ptr<BooleanEvalValue> EvaluateBoolExpression(const BoundExpression *expression);
        const shared_ptr<StringEvalValue> EvaluateStringExpression(const BoundExpression *expression);
        const shared_ptr<EvalValue> EvaluateFunctionExpression(const BoundExpression *expression);
        const shared_ptr<EvalValue> EvaluateNilExpression(const BoundExpression *expression);
        const shared_ptr<EvalValue> EvaluateTableExpression(const BoundExpression *expression);

        const shared_ptr<NumericEvalValue> EvaluateIntegerBinary(const BoundBinaryExpression *expression);
        const shared_ptr<BooleanEvalValue> EvaluateBooleanBinary(const BoundBinaryExpression *expression);
        const shared_ptr<StringEvalValue> EvaluateStringBinary(const BoundBinaryExpression *expression);

        const shared_ptr<NumericEvalValue> EvaluateIntegerUnary(const BoundUnaryExpression *expression);
        const shared_ptr<BooleanEvalValue> EvaluateBooleanUnary(const BoundUnaryExpression *expression);
        const shared_ptr<EvalValue> EvaluateFunctionCallExpression(const BoundExpression *expression);
        const shared_ptr<EvalValue> EvaluateIndexExpression(const BoundExpression *expression);
        const shared_ptr<EvalValue> EvaluatePeriodIndexExpression(const BoundExpression *expression);
        const shared_ptr<EvalValue> EvaluateNumericTableExpression(const BoundExpression *expression);
        const shared_ptr<EvalValue> EvaluateComplexTableExpression(const BoundExpression *expression);
        const shared_ptr<EvalValue> EvaluateUserDataExpression(const BoundExpression *expression);

        const shared_ptr<EvalValue> GetVariable(const BoundVariableExpression *expression);
    public:
        explicit Evaluator(unordered_map<uint32_t, shared_ptr<EvalValue>>* scriptVariables){
            _scriptVariables = scriptVariables;
            _hasReturned = false;
            _returnValue = nullptr;
            _evaluationScope = nullptr;
        }

        EvalValue* Evaluate(const BoundScriptStatement* statement);
        const shared_ptr<EvalValue> EvaluateFunction(const ScriptFunctionEvalValue *function,
                                                     const vector<EvalValue *> &parameters);

        EvalValue* GetLastValue(){
            return _lastValue.get();
        }

    };
}



#endif //PORYGONLANG_EVALUATOR_HPP
