
#include "EvalValues/NumericEvalValue.hpp"
#include "Evaluator.hpp"
#include "EvaluationException.hpp"
#include "../Script.hpp"

namespace Porygon::Evaluation {
    const shared_ptr<NumericEvalValue> Evaluator::EvaluateIntegerUnary(const BoundUnaryExpression *expression) {
        switch (expression->GetOperation()) {
            case BoundUnaryOperation::Negation: {
                auto operandValue = EvaluateIntegerExpression(expression->GetOperand());
                if (operandValue->IsFloat()) {
                    double f = operandValue->EvaluateFloat();
                    return make_shared<FloatEvalValue>(-f);
                } else {
                    long l = operandValue->EvaluateInteger();
                    return make_shared<IntegerEvalValue>(-l);
                }
            }
            case BoundUnaryOperation::LogicalNegation:
                throw;
        }
    }

    const shared_ptr<BooleanEvalValue> Evaluator::EvaluateBooleanUnary(const BoundUnaryExpression *expression) {
        switch (expression->GetOperation()) {
            case BoundUnaryOperation::LogicalNegation: {
                auto val = EvaluateBoolExpression(expression->GetOperand());
                bool b = val->EvaluateBool();
                return make_shared<BooleanEvalValue>(!b);
            }
            case BoundUnaryOperation::Negation:
                throw;
        }
    }
}
