#include <utility>

#ifndef PORYGONLANG_SCRIPT_HPP
#define PORYGONLANG_SCRIPT_HPP

#include <utility>
#include <string>
#include <unordered_map>
#include "Diagnostics/DiagnosticsHolder.hpp"
#include "Binder/BoundStatements/BoundStatement.hpp"
#include "Evaluator/Evaluator.hpp"
#include "Evaluator/EvalValues/EvalValue.hpp"
#include "Utilities/HashedString.hpp"

using namespace std;
using namespace Porygon::Evaluation;

namespace Porygon{
    class Script {
        Evaluator* _evaluator;
        unordered_map<uint32_t, shared_ptr<EvalValue>>* _scriptVariables;
        Binder::BoundScriptStatement* _boundScript;
        shared_ptr<ScriptType> _returnType;

        explicit Script();
        void Parse(const u16string& script);
    public:
        static Script* Create(const u16string& script);
        static Script* Create(const string& script);
        DiagnosticsHolder* Diagnostics;

        ~Script();

        shared_ptr<ScriptType> GetReturnType(){
            return _returnType;
        }

        void SetReturnType(shared_ptr<ScriptType> t){
            _returnType  = std::move(t);
        }

        EvalValue* Evaluate();

        EvalValue* GetLastValue();

        EvalValue* GetVariable(const u16string& key);
        bool HasVariable(const u16string& key);

        shared_ptr<EvalValue> CallFunction(const u16string& key, const vector<EvalValue*>& variables);
        bool HasFunction(const u16string& key);
    };

}


#endif //PORYGONLANG_SCRIPT_HPP
