

#ifndef PORYGONLANG_USERDATASCRIPTTYPE_HPP
#define PORYGONLANG_USERDATASCRIPTTYPE_HPP

#include <utility>
#include "../ScriptType.hpp"
#include "UserData.hpp"
#include "UserDataStorage.hpp"

namespace Porygon::UserData {
    class UserDataScriptType : public ScriptType {
        shared_ptr<UserData> _userData;
    public:
        explicit UserDataScriptType(uint32_t id) : ScriptType(TypeClass::UserData) {
            _userData = UserDataStorage::GetUserDataType(id);
        }

        explicit UserDataScriptType(shared_ptr<UserData> ud) : ScriptType(TypeClass::UserData) {
            _userData = std::move(ud);
        }

        const bool CanBeIndexedWith(ScriptType *indexer) const final {
            if (indexer->GetClass() != TypeClass::String) {
                return false;
            }
            auto str = (StringScriptType *) indexer;
            if (!str->IsKnownAtBind())
                return false;
            return _userData->ContainsField(str->GetHashValue());
        }

        const bool CanBeIndexedWithIdentifier(uint32_t hash) const final {
            return true;
        }

        UserDataField *GetField(uint32_t id) {
            return _userData->GetField(id);
        }

        const shared_ptr<ScriptType> GetIndexedType(ScriptType *indexer) const final {
            auto stringKey = (StringScriptType *) indexer;
            if (stringKey->IsKnownAtBind()) {
                return _userData->GetField(stringKey->GetHashValue())->GetType();
            }
            throw "TODO: indexing with dynamic keys";
        }

        const shared_ptr<ScriptType> GetIndexedType(uint32_t hash) const final {
            return _userData->GetField(hash)->GetType();
        }
    };
}


#endif //PORYGONLANG_USERDATASCRIPTTYPE_HPP
